#ifndef COORD_H
#define COORD_H

#include <cmath>

enum Dir {
    NORTH,
    EAST,
    SOUTH,
    WEST,
    NO_DIR = -1
};

struct Coord
{
    int x;
    int y;
    
    Coord(const Coord &c) :
        x(c.x), y(c.y) {}
    Coord(int x, int y) :
        x(x), y(y) {}
    Coord(int val) :
        x(val), y(val) {}
    Coord(Dir d = NO_DIR)
    {
        switch(d)
        {
            case NORTH : x= 0; y=-1; break;
            case SOUTH : x= 0; y= 1; break;
            case WEST  : x=-1; y= 0; break;
            case EAST  : x= 1; y= 0; break;
            default    : x= 0; y= 0; break;
        }
    }
    bool operator==(const Coord &c) const
        { return x==c.x&&y==c.y;}
    Coord operator+(const Coord &c) const
        { return Coord(x+c.x, y+c.y);}
    Coord& operator+=(const Coord &c)
        { x+=c.x; y+=c.y; return *this; }
    Coord operator+(Dir d) const
        { return *this + Coord(d); }
    Coord& operator+=(Dir d)
        { return *this += Coord(d); }
    Coord operator-(const Coord &c) const
        { return Coord(x-c.x, y-c.y); }
    Coord& operator-=(const Coord &c)
        { x-=c.x; y-=c.y; return *this; }
    Coord operator-(Dir d) const
        { return *this - Coord(d); }
    Coord& operator-=(Dir d)
        { return *this -= Coord(d); }
    Coord operator*(const Coord &c) const
        { return Coord(x*c.x, y*c.y); }
    Coord& operator*=(const Coord &c)
        { x*=c.x; y*=c.y; return *this; }
    Coord operator/(const Coord &c) const
        { return Coord(x/c.x, y/c.y); }
    bool operator<(const Coord &c) const // less comparator is used for sorted containers (std::set, std::map, ...)
        { if(x < c.x) return true; else return y < c.y; }
    Coord& operator/=(const Coord &c)
        { x/=c.x; y/=c.y; return *this; }
    int dist() const // manhatthan distance
        { return std::abs(x) + std::abs(y); }
    int operator~() const
        { return dist(); }
    static int dist(const Coord &c1, const Coord &c2)
        { return ~(c1-c2); }
    int dist(const Coord &c) const
        { return dist(*this, c); }
    int dist(int x, int y) const
        { return dist(*this, Coord(x, y)); }

};

struct CoordHash{
    std::size_t operator()(const Coord& val) const{
        return val.x+10000*val.y;
    }
};

#endif // COORD_H
