#ifndef SIMULATION_H
#define SIMULATION_H

#include <vector>
#include <set>
#include <tuple>
#include "behavior.h"
#include "team.h"

class MapScene;

class Simulation
{
private:
    typedef std::pair<Dude*,Dude*> Battle;
    struct battleComparator{
        bool operator() (Battle a, Battle b){
            return ((a == b) || (a.first == b.second && b.first == a.second));
        }
    };

    MapScene *p_map;
    std::vector<Dude*> m_dudes;
    std::vector<Team> m_teams;
    std::set<Battle,battleComparator> m_battles;

    void handleAction(Dude* dude);
    void resolveBattles();

public:
    Simulation(MapScene *_map, std::vector<BehaviorFunction> &_behaviors);
    ~Simulation();
    MapScene* getMap() { return (MapScene*) p_map; }
    int getPopulation(){return m_dudes.size();}

    /**
     * @brief update runs one step of simulation
     */
    void update();
};

#endif // SIMULATION_H
